package com.house365.xxx.house.word.search.core;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.annotation.Resource;

import com.house365.xxx.house.word.search.CityAndDBManager;
import org.apache.commons.lang.StringUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.stereotype.Component;

import com.house365.xxx.house.word.search.dto.WordBean;
import com.house365.xxx.house.word.search.dto.WordRatio;
import com.house365.xxx.house.word.search.sell.BlockAliasLoader;
import com.house365.xxx.house.word.search.sell.BlockLoader;
import com.house365.xxx.house.word.search.sell.CommunityLoader;
import com.house365.xxx.house.word.search.sell.DistrictLoader;
import com.house365.xxx.house.word.search.sell.SchoolLoader;
import com.house365.xxx.house.word.search.sell.StreetLoader;
import com.house365.xxx.house.word.search.sell.TrafficLoader;
import com.house365.xxx.house.word.search.util.PinyinToolkit;

@Component
public class DistanceChecker {
	private static Log logger = LogFactory.getLog(DistanceChecker.class);
	@Resource
	private BlockLoader blockLoader;
	@Resource
	private SchoolLoader schoolLoader;
	@Resource
	private TrafficLoader trafficLoader;
	@Resource
	private CommunityLoader communityLoader;
	@Resource
	private BlockAliasLoader blockAliasLoader;
	@Resource
    private DistrictLoader districtLoader;
	@Resource
	private StreetLoader streetLoader;
	@Resource
	protected CityAndDBManager cityAndDB;

	public void init() {
	    logger.info("DistanceChecker init start");
		Set<String> citys = cityAndDB.getSellCitys();
		blockLoader.loadWordBeans(citys);
		schoolLoader.loadWordBeans(citys);
		trafficLoader.loadWordBeans(citys);
		communityLoader.loadWordBeans(citys);
		blockAliasLoader.loadWordBeans(citys);
		districtLoader.loadWordBeans(citys);
		streetLoader.loadWordBeans(citys);
		logger.info("DistanceChecker init end");
	}

	public List<WordRatio> getWordsFromBlock(String city, String word) {
		List<WordRatio> lstWR = getWordRatiosFromBlock(city, word);
		Collections.sort(lstWR);
		int size = 20 > lstWR.size() ? lstWR.size() : 20;
		List<WordRatio> lstRS = lstWR.subList(0, size);
		return lstRS;
	}

    public List<WordRatio> getWordsFromSell(String city, String word, final String biz) {
        List<WordRatio> lstWR = getWordRatiosFromBlock(city, word);
        if (StringUtils.equals(biz, "sell")) {
            lstWR.addAll(getWordRatiosFromSchool(city, word));
        }
        lstWR.addAll(getWordRatiosFromTraffic(city, word));
        lstWR.addAll(getWordRatiosFromCommunity(city, word));
        lstWR.addAll(getWordRatiosFromBlockAlias(city, word));
        lstWR.addAll(getWordRatiosFromDistrictByEquals(city, word));
        lstWR.addAll(getWordRatiosFromStreetByEquals(city, word));
        // WordBean的type id相同的只保留一个
        lstWR = distinctWR(lstWR);

        Collections.sort(lstWR, new Comparator<WordRatio>() {
            public int compare(WordRatio wr1, WordRatio wr2) {
                // 联想推荐排序规则 一级 区属>>街道>>社区>>小区>>地铁站>>学校
                if (wr1.getWordBean().getDicOrder() < wr2.getWordBean().getDicOrder()) {
                    return 1;
                } else if (wr1.getWordBean().getDicOrder() > wr2.getWordBean().getDicOrder()) {
                    return -1;
                } else {
                    // 联想推荐排序规则 二级 房源数量多>>房源数量少
                    if (StringUtils.equals(biz, "sell")) {
                        if (wr1.getWordBean().getSellCount() < wr2.getWordBean().getSellCount()) {
                            return 1;
                        } else if (wr1.getWordBean().getSellCount() > wr2.getWordBean().getSellCount()) {
                            return -1;
                        } else {
                            return 0;
                        }
                    } else if (StringUtils.equals(biz, "rent")) {
                        if (wr1.getWordBean().getRentCount() < wr2.getWordBean().getRentCount()) {
                            return 1;
                        } else if (wr1.getWordBean().getRentCount() > wr2.getWordBean().getRentCount()) {
                            return -1;
                        } else {
                            return 0;
                        }
                    } else {
                        // 联想推荐排序规则 三级 相似度
                        if (wr1.getRatio() < wr2.getRatio()) {
                            return 1;
                        } else if (wr1.getRatio() > wr2.getRatio()) {
                            return -1;
                        }
                    }
                }

                return 0;
            }
        });
        return lstWR;
    }

    private List<WordRatio> distinctWR(List<WordRatio> lstWR) {

        Set<String> keySet = new HashSet<>();
        List<WordRatio> result = new ArrayList<>();
        for (WordRatio wr : lstWR) {
            int type = wr.getWordBean().getType().getValue();
            int id = wr.getWordBean().getId();
            String key = "" + type + "@" + id;
            if (!keySet.contains(key)) {
                result.add(wr);
                keySet.add(key);
            }
        }
        return result;
    }

    private List<WordRatio> getWordRatiosFromBlockAlias(String city, String word) {
        Set<WordBean> wordBeans = blockAliasLoader.getWordBeanByCity(city);
        if (wordBeans != null && !wordBeans.isEmpty()) {
            List<WordRatio> initialResult = findWordsByContains(word, wordBeans);
            List<WordRatio> finalResult = exchangeNameAlias(initialResult);
            return finalResult;
        }
        return new ArrayList<WordRatio>();
    }

    private List<WordRatio> exchangeNameAlias(List<WordRatio> initialResult) {
        List<WordRatio> result = new ArrayList<>();
        for (WordRatio wr : initialResult) {
            WordBean oldWb = wr.getWordBean();
            WordBean newWb = new WordBean(oldWb.getId(), oldWb.getAlias(), oldWb.getPinyin(),
                    oldWb.getType(), oldWb.getDistrictStreet(), oldWb.getSellCount(),
                    oldWb.getRentCount(), oldWb.getWord(), oldWb.getExtInfo(), oldWb.getAverPrice());
            WordRatio newWr = new WordRatio(newWb, wr.getRatio());
            result.add(newWr);
        }
        return result;
    }

    private List<WordRatio> getWordRatiosFromCommunity(String city, String word) {
        Set<WordBean> wordBeans = communityLoader.getWordBeanByCity(city);
        if (wordBeans != null && !wordBeans.isEmpty()) {
            return findWordsByContains(word, wordBeans);
        }
        return new ArrayList<WordRatio>();
    }

	private List<WordRatio> getWordRatiosFromBlock(String city, String word) {
		Set<WordBean> wordBeans = blockLoader.getWordBeanByCity(city);
		if (wordBeans != null && !wordBeans.isEmpty()) {
			return findWordsByContains(word, wordBeans);
		}
		return new ArrayList<WordRatio>();
	}

	private List<WordRatio> getWordRatiosFromSchool(String city, String word) {
		Set<WordBean> wordBeans = schoolLoader.getWordBeanByCity(city);
		if (wordBeans != null && !wordBeans.isEmpty()) {
			return findWordsByContains(word, wordBeans);
		}
		return new ArrayList<WordRatio>();
	}

	private List<WordRatio> getWordRatiosFromTraffic(String city, String word) {
		Set<WordBean> wordBeans = trafficLoader.getWordBeanByCity(city);
		if (wordBeans != null && !wordBeans.isEmpty()) {
			return findWordsByContains(word, wordBeans);
		}
		return new ArrayList<WordRatio>();
	}

	public List<WordRatio> getWordsFromSellByEquals(String city, String word) {
		List<WordRatio> lstWR = getWordRatiosFromBlockByEquals(city, word);
		lstWR.addAll(getWordRatiosFromSchoolByEquals(city, word));
		lstWR.addAll(getWordRatiosFromTrafficByEquals(city, word));
		lstWR.addAll(getWordRatiosFromCommunityByEquals(city, word));
		lstWR.addAll(getWordRatiosFromBlockAliasByEquals(city, word));
		lstWR.addAll(getWordRatiosFromDistrictByEquals(city, word));
		lstWR.addAll(getWordRatiosFromStreetByEquals(city, word));
		return lstWR;
	}

	private List<WordRatio> getWordRatiosFromBlockAliasByEquals(String city,
            String word) {
        Set<WordBean> wordBeans = blockAliasLoader.getWordBeanByCity(city);
        if (wordBeans != null && !wordBeans.isEmpty()) {
            return findWordsByEquals(word, wordBeans);
        }
        return new ArrayList<WordRatio>();
    }

    private List<WordRatio> getWordRatiosFromCommunityByEquals(String city,
            String word) {
        Set<WordBean> wordBeans = communityLoader.getWordBeanByCity(city);
        if (wordBeans != null && !wordBeans.isEmpty()) {
            return findWordsByEquals(word, wordBeans);
        }
        return new ArrayList<WordRatio>();
    }

    private List<WordRatio> getWordRatiosFromBlockByEquals(String city,
			String word) {
		Set<WordBean> wordBeans = blockLoader.getWordBeanByCity(city);
		if (wordBeans != null && !wordBeans.isEmpty()) {
			return findWordsByEquals(word, wordBeans);
		}
		return new ArrayList<WordRatio>();
	}

	private List<WordRatio> getWordRatiosFromSchoolByEquals(String city,
			String word) {
		Set<WordBean> wordBeans = schoolLoader.getWordBeanByCity(city);
		if (wordBeans != null && !wordBeans.isEmpty()) {
			return findWordsByEquals(word, wordBeans);
		}
		return new ArrayList<WordRatio>();
	}

	private List<WordRatio> getWordRatiosFromTrafficByEquals(String city,
			String word) {
		Set<WordBean> wordBeans = trafficLoader.getWordBeanByCity(city);
		if (wordBeans != null && !wordBeans.isEmpty()) {
			return findWordsByEquals(word, wordBeans);
		}
		return new ArrayList<WordRatio>();
	}
	
	private List<WordRatio> getWordRatiosFromDistrictByEquals(String city,String word){
		Set<WordBean> wordBeans = districtLoader.getWordBeanByCity(city);
		if (wordBeans != null && !wordBeans.isEmpty()) {
			return findWordsByContains(word, wordBeans);
		}
		return new ArrayList<WordRatio>();
	}
	
	private List<WordRatio> getWordRatiosFromStreetByEquals(String city,String word){
		Set<WordBean> wordBeans = streetLoader.getWordBeanByCity(city);
		if (wordBeans != null && !wordBeans.isEmpty()) {
			return findWordsByContains(word, wordBeans);
		}
		return new ArrayList<WordRatio>();
	}

	// 编辑距离计算
	// private List<WordRatio> findWordsByWordDis(String word, Set<WordBean>
	// dic) {
	// List<WordRatio> lstRs = new ArrayList<WordRatio>();
	// for (WordBean wordBean : dic) {
	// float dis = 0;
	// String lw = wordBean.getWord();
	// float dicLen = lw.length();
	// float myLen = word.length();
	// dis = DistanceCalculate.getWordDistance(word, lw);
	// float len = myLen > dicLen ? myLen : dicLen;
	// float matchSize = len - dis;
	// float matchRate = (matchSize / dicLen + matchSize / myLen) / 2;
	// if (matchRate > 0.2)
	// lstRs.add(new WordRatio(wordBean, matchRate));
	// }
	// return lstRs;
	// }
	public static boolean isChineseChar(String str) {
		boolean temp = false;
		Pattern p = Pattern.compile("[\u4e00-\u9fa5]");
		Matcher m = p.matcher(str);
		if (m.find()) {
			temp = true;
		}
		return temp;
	}

	private List<WordRatio> findWordsByContains(String word, Set<WordBean> dic) {
		List<WordRatio> lstRs = new ArrayList<WordRatio>();
		for (WordBean wordBean : dic) {
			String lw = wordBean.getWord();
			if (isChineseChar(word)) {
				if (lw.contains(word)) {
					float dicLen = lw.length();
					float myLen = word.length();
					float matchRate = myLen / dicLen;
					lstRs.add(new WordRatio(wordBean, matchRate));
				}
			} else {
				String dicPY = wordBean.getPinyin();
				if (StringUtils.isNotBlank(dicPY)) {
					String wordPY = PinyinToolkit.cn2Spell(word);
					// logger.info("wordPY:" + wordPY + " dicPY:" + dicPY);
					if (dicPY.startsWith(wordPY)) {
						float dicLen = dicPY.length();
						float myLen = wordPY.length();
						float matchRate = myLen / dicLen;
						lstRs.add(new WordRatio(wordBean, matchRate));
					}
				}
			}
		}
		return lstRs;
	}

	private List<WordRatio> findWordsByEquals(String word, Set<WordBean> dic) {
		List<WordRatio> lstRs = new ArrayList<WordRatio>();
		for (WordBean wordBean : dic) {
			String lw = wordBean.getWord();
			if (lw.equalsIgnoreCase(word)) {
				lstRs.add(new WordRatio(wordBean, 1.0f));
			}
		}
		return lstRs;
	}
}
