package com.house365.xxx.house.word.search.sell;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.stereotype.Component;

import com.house365.xxx.house.word.search.DicType;
import com.house365.xxx.house.word.search.dto.DistrictStreet;
import com.house365.xxx.house.word.search.dto.WordBean;
import com.house365.xxx.house.word.search.util.PinyinToolkit;

/**
 * 二手房小区别名
 */
@Component
public class BlockAliasLoader extends SellLoader {

	@Override
	protected DicType getDicType() {
		return DicType.BLOCK;
	}

	@Override
	protected String getLoadSql() {
		return "SELECT\r\n" + 
				"	t_block.id,\r\n" + 
				"	t_block.blockname,\r\n" + 
				"	t_block.district,\r\n" + 
				"	t_street.streetname,\r\n" + 
				"	ifnull(t_block_count.sellcount, 0) AS sellcount,\r\n" + 
				"	ifnull(t_block_count.rentcount, 0) AS rentcount,\r\n" + 
				"	t_block.alias,\r\n" + 
				"	'' AS extinfo,\r\n" + 
				"	t_block.averprice\r\n" + 
				"FROM\r\n" + 
				"	block t_block\r\n" + 
				"LEFT OUTER JOIN street t_street ON t_block.streetid = t_street.id\r\n" + 
				"LEFT OUTER JOIN block_count t_block_count ON t_block.id = t_block_count.blockid\r\n" + 
				"WHERE\r\n" + 
				"	t_block.esta = 2\r\n" + 
				"AND t_block.alias != ''";
	}

	@Override
	protected Log getLogger() {
		return LogFactory.getLog(BlockAliasLoader.class);
	}

    @Override
    protected WordBean getWordBean(int id, String name, String pinyin, DicType dicType,
            DistrictStreet districtStreet, int sellCount, int rentCount, String alias, String extInfo, String averPrice) {
        WordBean wordBean = new WordBean(id, alias, PinyinToolkit.cn2Spell(alias), getDicType(),
                districtStreet, sellCount, rentCount, name, extInfo, averPrice);
        return wordBean;
    }

}
